unit unBackgroundsDM;

interface

uses
  SysUtils, Classes, DB, IBCustomDataSet, IBStoredProc, IBDatabase, IBQuery,
  ExtCtrls;

type
  TSetNewGenEvent = procedure( Sender: TObject; ChangesCount: Integer ) of object;

  TBackgroundsDM = class(TDataModule)
    Database: TIBDatabase;
    WriteTransaction: TIBTransaction;
    SetNewGenProc: TIBStoredProc;
    ClearDelLogsProc: TIBStoredProc;
    ReadQuery: TIBQuery;
    TrySetNewGenTimer: TTimer;
    ClearLogsTimer: TTimer;
    ReadTransaction: TIBTransaction;
    procedure TrySetNewGenTimerTimer(Sender: TObject);
    procedure ClearLogsTimerTimer(Sender: TObject);
  private
    { Private declarations }
    FSetNewGenTriesCount: Integer;
    FMaxSetNewGenTries:   Integer;
    FMaxChangesCount:     Integer;
    FActive:            Boolean;
    FOnAfterClearLogs: TNotifyEvent;
    FOnAfterTrySetNewGen: TSetNewGenEvent;
    function    GetClearLogsInterval: Cardinal;
    function    GetTrySetNewGenInterval: Cardinal;
    procedure   SetClearLogsInterval( const Value: Cardinal );
    procedure   SetTrySetNewGenInterval( const Value: Cardinal );
    procedure   SetActive(const Value: Boolean);
    procedure   SetOnAfterClearLogs(const Value: TNotifyEvent);
    procedure   SetOnAfterTrySetNewGen(const Value: TSetNewGenEvent);
    function    GetDatabaseName: string;
    procedure   SetNewGen;
    function GetOpened: Boolean;
 public
    { Public declarations }
    property    SetNewGenTriesCount: Integer read FSetNewGenTriesCount;
    property    MaxSetNewGenTries: Integer read FMaxSetNewGenTries write FMaxSetNewGenTries;
    property    MaxChangesCount:   Integer read FMaxChangesCount   write FMaxChangesCount;
    property    Active: Boolean read FActive write SetActive;
    property    TrySetNewGenInterval: Cardinal read GetTrySetNewGenInterval write SetTrySetNewGenInterval;
    property    ClearLogsInterval: Cardinal read GetClearLogsInterval write SetClearLogsInterval;
    property    DatabaseName: string read GetDatabaseName;
    property    OnAfterTrySetNewGen: TSetNewGenEvent read FOnAfterTrySetNewGen write SetOnAfterTrySetNewGen;
    property    OnAfterClearLogs: TNotifyEvent read FOnAfterClearLogs write SetOnAfterClearLogs;
    procedure   Open( const DatabaseName: string; Activate: Boolean = True );
    property    Opened: Boolean read GetOpened;
    procedure   Close;
    function    GetCurrDBID: Integer;
    function    GetCurrGen: Integer;
    function    TrySetNewGen( Force: Boolean = False ): Boolean;
    procedure   ClearLogs;
    function    GetChangesCount: Int64;
    constructor Create( AOwner: TComponent );override;
  end;

var
  BackgroundsDM: TBackgroundsDM;

implementation

{$R *.dfm}

{ TBackgroundsDM }

const
  Second = 1000;
  Minute = 60*Second;
  Hour   = 60*Minute;

constructor TBackgroundsDM.Create( AOwner: TComponent );
  begin
    inherited Create( AOwner );
    TrySetNewGenInterval  := 5*Second;
    ClearLogsInterval     := 30*Second;
    MaxSetNewGenTries     := 10;
    MaxChangesCount       := 10;
    FActive              := False;
    FSetNewGenTriesCount  := 0;
  end;

function TBackgroundsDM.GetCurrDBID: Integer;
  const
    SQL: string = 'select DB_ID from CURR_DB_ID';
  begin
    ReadQuery.SQL.Text := SQL;
    ReadTransaction.StartTransaction;
    try
      ReadQuery.Open;
      try
        ReadQuery.First;
        Result := ReadQuery.FieldByName( 'DB_ID' ).AsInteger;
        ReadQuery.Next;
        if not ReadQuery.Eof then
            raise Exception.Create( 'TBackgroundsDM.GetCurrDBID' );
      finally
        ReadQuery.Close;
      end;
      ReadTransaction.Commit;
    except
      ReadTransaction.Rollback;
      raise;
    end;
  end;


function TBackgroundsDM.GetCurrGen: Integer;
  const
    SQL: string = 'select GEN from CURR_GEN';
  begin
    ReadQuery.SQL.Text := SQL;
    ReadTransaction.StartTransaction;
    try
      ReadQuery.Open;
      try
        ReadQuery.First;
        Result := ReadQuery.FieldByName( 'GEN' ).AsInteger;
        ReadQuery.Next;
        if not ReadQuery.Eof then
            raise Exception.Create( 'TBackgroundsDM.GetCurrGen' );
      finally
        ReadQuery.Close;
      end;
      ReadTransaction.Commit;
    except
      ReadTransaction.Rollback;
      raise;
    end;
  end;


procedure TBackgroundsDM.ClearLogs;
  begin
    WriteTransaction.StartTransaction;
    try
      ClearDelLogsProc.ExecProc;
      WriteTransaction.Commit;
    except
      WriteTransaction.Rollback;
      raise;
    end;
    if Assigned( OnAfterClearLogs ) then
      OnAfterClearLogs( self );
   end;

procedure TBackgroundsDM.SetNewGen;
  begin
    WriteTransaction.StartTransaction;
    try
      SetNewGenProc.ExecProc;
      //Sleep( 1000 );
      WriteTransaction.Commit;
    except
      WriteTransaction.Rollback;
      raise;
    end;
  end;


procedure TBackgroundsDM.Close;
  begin
    Database.Close;
  end;

procedure TBackgroundsDM.Open( const DatabaseName: string; Activate: Boolean = True );
  begin
    Database.Close;
    Database.DatabaseName := DatabaseName;
    Database.Open;
    Active := Activate;
  end;


function TBackgroundsDM.GetChangesCount: Int64;
  const
    SQL: string = 'select CHANGES_COUNT from CHANGES_COUNT';
  begin
    ReadQuery.SQL.Text := SQL;
    ReadTransaction.StartTransaction;
    try
      ReadQuery.Open;
      try
        ReadQuery.First;
        Result := ReadQuery.FieldByName( 'CHANGES_COUNT' ).AsInteger;
        ReadQuery.Next;
        if not ReadQuery.Eof then
            raise Exception.Create( 'TBackgroundsDM.GetChangesCount' );
      finally
        ReadQuery.Close;
      end;
      ReadTransaction.Commit;
    except
      ReadTransaction.Rollback;
      raise;
    end;
  end;


function TBackgroundsDM.TrySetNewGen( Force: Boolean = False ): Boolean;
  var
    ChangesCount: Integer;
  begin
    ChangesCount := GetChangesCount;
    if ( ChangesCount > 0 )and( SetNewGenTriesCount > MaxSetNewGenTries )
       or ( ChangesCount > MaxChangesCount )
       or Force
      then
        begin
          SetNewGen;
          FSetNewGenTriesCount := 0;
          Result := True;
        end
      else
        begin
          inc( FSetNewGenTriesCount );
          Result := False;
        end;
    if Assigned( OnAfterTrySetNewGen ) then
      OnAfterTrySetNewGen( Self, ChangesCount );
  end;


function TBackgroundsDM.GetClearLogsInterval: Cardinal;
  begin
    Result := ClearLogsTimer.Interval;
  end;

procedure TBackgroundsDM.SetClearLogsInterval( const Value: Cardinal );
  begin
    ClearLogsTimer.Interval := Value;
  end;

function TBackgroundsDM.GetTrySetNewGenInterval: Cardinal;
  begin
    Result := TrySetNewGenTimer.Interval;
  end;

procedure TBackgroundsDM.SetTrySetNewGenInterval( const Value: Cardinal );
  begin
    TrySetNewGenTimer.Interval := Value;
  end;

procedure TBackgroundsDM.SetActive( const Value: Boolean );
  begin
    FActive := Value;
    TrySetNewGenTimer.Enabled := Value;
    ClearLogsTimer.Enabled := Value;
  end;

procedure TBackgroundsDM.TrySetNewGenTimerTimer(Sender: TObject);
  begin
    TrySetNewGen;
  end;

procedure TBackgroundsDM.ClearLogsTimerTimer(Sender: TObject);
  begin
    ClearLogs;
  end;


procedure TBackgroundsDM.SetOnAfterClearLogs(const Value: TNotifyEvent);
  begin
    FOnAfterClearLogs := Value;
  end;


procedure TBackgroundsDM.SetOnAfterTrySetNewGen(const Value: TSetNewGenEvent);
  begin
    FOnAfterTrySetNewGen := Value;
  end;

function TBackgroundsDM.GetDatabaseName: string;
  begin
    Result := Database.DatabaseName;
  end;

function TBackgroundsDM.GetOpened: Boolean;
  begin
    Result := Database.Connected;
  end;

end.
